 /**
  ******************************************************************************
  * @file    Template/Src/main.c 
  * @author  MCD Application Team
  * @version V0.3.0
  * @date    09-March-2015
  * @brief   STM32F7xx HAL API Template project 
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2015 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32746g_discovery.h"
#include "stm32746g_discovery_sdram.h"
#include  "lcd_log.h"

/** @addtogroup STM32F7xx_HAL_Examples
  * @{
  */

/** @addtogroup Templates
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define NUM_SAMPLES 500
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
static void SystemClock_Config(void);
static void Error_Handler(void);
static void MPU_Config(void);
static void CPU_CACHE_Enable(void);
static void printConfig(void);

extern volatile uint8_t tst[16];
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
	
volatile uint32_t result[4];

uint32_t array_sdram[NUM_SAMPLES] __attribute__((at(0xC0000000)));   // in external SDRAM
uint32_t array_dtcm[NUM_SAMPLES];  // in internal DTCM SRAM

uint32_t time_start, time_end;
volatile uint32_t time_diff;
int main(void)
{
  uint32_t n, acc;
  
  HAL_Init();
  
  SystemClock_Config();
 
  BSP_SDRAM_Init();
  // initialize the array with some values
  for (n = 0; n < NUM_SAMPLES; n++)
  {
    array_sdram[n] = n;
    array_dtcm[n] = n;
  }
  result[0] = result[1] = result[2] = result[3] = 0;
  
  MPU_Config(); // configure the memory region
  printConfig();
  printf("->CPU cycles spent for sum of %d values \n", NUM_SAMPLES);
  HAL_SYSTICK_Config(0xFFFFFF);
  
  acc = 0;
  // DTCM data
  TIM_MEASURE_START;
  for (n = 0; n < NUM_SAMPLES; n++)
  {
    acc += (array_dtcm[n]);
  }
  TIM_MEASURE_END;
  result[0] = acc;   // read out the accumulator into volatile variable in order to not loose the content
  printf(" DTCM data (array_dtcm):                   %d cycles\n", time_diff);

  acc = 0;
  // No D-Cache - external memory
  TIM_MEASURE_START;
  for (n = 0; n < NUM_SAMPLES; n++)
  {
    acc += (array_sdram[n]);
  }
  TIM_MEASURE_END;
  result[1] = acc;   // read out the accumulator into volatile variable in order to not loose the content
  printf(" D-Cache disabled (array_sdram):            %d cycles\n", time_diff);
  // BSP_LCD_DisplayOn();
	
  // enable the D-Cache now
  SCB_EnableDCache();

  acc = 0;
  // D-Cache Enabled - first run
  TIM_MEASURE_START;
  for (n = 0; n < NUM_SAMPLES; n++)
  {
    acc += (array_sdram[n]);
  }
  TIM_MEASURE_END;
  result[2] = acc;   // read out the accumulator into volatile variable in order to not loose the content
  printf(" D-Cache enabled, first run (array_sdram):  %d cycles\n", time_diff);
  
  acc = 0;
  // D-Cache Enabled - second run
  TIM_MEASURE_START;
  for (n = 0; n < NUM_SAMPLES; n++)
  {
    acc += (array_sdram[n]);
  }
  TIM_MEASURE_END;
  result[3] = acc;   // read out the accumulator into volatile variable in order to not loose the content
  printf(" D-Cache enabled, second run (array_sdram): %d cycles\n", time_diff);

  printf(" results of computation are : %d,%d,%d,%d\n", result[0],result[1],result[2],result[3]);
  BSP_LCD_DisplayOn();
  while(1);
	
}


/**
  * @brief  Configure the MPU attributes as Write Through for SRAM1/2.
  * @note   The Base Address is 0x20010000 since this memory interface is the AXI.
  *         The Region Size is 256KB, it is related to SRAM1 and SRAM2  memory size.
  * @param  None
  * @retval None
  */
#ifdef ARM_MATH_CM7
static void MPU_Config(void)
{
  MPU_Region_InitTypeDef MPU_InitStruct;
  
  /* Disable the MPU */
  HAL_MPU_Disable();

  /* Configure the MPU attributes as WT for SRAM */
  MPU_InitStruct.Enable = MPU_REGION_ENABLE;
  MPU_InitStruct.BaseAddress = 0xC0000000;
  MPU_InitStruct.Size = MPU_REGION_SIZE_128KB;
  MPU_InitStruct.AccessPermission = MPU_REGION_FULL_ACCESS;
  MPU_InitStruct.IsShareable = MPU_ACCESS_NOT_SHAREABLE;
  MPU_InitStruct.Number = MPU_REGION_NUMBER0;
  MPU_InitStruct.SubRegionDisable = 0x00;
  MPU_InitStruct.DisableExec = MPU_INSTRUCTION_ACCESS_DISABLE;

#define DCACHE_WBWA

#ifdef  DCACHE_WBWA
  // these 3 parameters configure Cache properties for this region - write back, write allocate
  MPU_InitStruct.IsBufferable = MPU_ACCESS_BUFFERABLE;
  MPU_InitStruct.IsCacheable = MPU_ACCESS_CACHEABLE;
  MPU_InitStruct.TypeExtField = MPU_TEX_LEVEL1;
#else  
  // these 3 parameters configure Cache properties for this region - write through
  MPU_InitStruct.IsBufferable = MPU_ACCESS_NOT_BUFFERABLE;
  MPU_InitStruct.IsCacheable = MPU_ACCESS_CACHEABLE;
  MPU_InitStruct.TypeExtField = MPU_TEX_LEVEL0;
#endif
  HAL_MPU_ConfigRegion(&MPU_InitStruct);

  /* Enable the MPU */
  HAL_MPU_Enable(MPU_PRIVILEGED_DEFAULT);
}
#endif
/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow : 
  *            System Clock source            = PLL (HSE)
  *            SYSCLK(Hz)                     = 200000000
  *            HCLK(Hz)                       = 200000000
  *            AHB Prescaler                  = 1
  *            APB1 Prescaler                 = 4
  *            APB2 Prescaler                 = 2
  *            HSE Frequency(Hz)              = 25000000
  *            PLL_M                          = 25
  *            PLL_N                          = 400
  *            PLL_P                          = 2
  *            PLL_Q                          = 8
  *            VDD(V)                         = 3.3
  *            Main regulator output voltage  = Scale1 mode
  *            Flash Latency(WS)              = 6
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_OFF;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
#if defined(STM32F756xx) || defined(STM32F746xx)
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 400;
#else
  RCC_OscInitStruct.PLL.PLLM = 8;
  RCC_OscInitStruct.PLL.PLLN = 168*2;
#endif
 
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 8;
  if(HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

#if defined(STM32F756xx) || defined(STM32F746xx)
  /* activate the OverDrive to reach the 200 Mhz Frequency */
  if(HAL_PWREx_EnableOverDrive() != HAL_OK)
  {
    Error_Handler();
  }
#endif
  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 
     clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;  
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;  
  if(HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_6) != HAL_OK)
  {
    Error_Handler();
  }
}


void printConfig(void)
{
  TPI->ACPR = (HAL_RCC_GetSysClockFreq() / 2000000) - 1; // make the SWO working whatever settings in the system frequency - output is always 2MHz
  
#ifdef ARM_MATH_CM7
  // LCD initialization
  BSP_LCD_Init();
  BSP_LCD_LayerDefaultInit(0, LCD_FB_START_ADDRESS); 
  BSP_LCD_SelectLayer(0);
  BSP_LCD_DisplayOff();
  LCD_LOG_Init();
  LCD_LOG_SetHeader((uint8_t *)"Data Cache example");
#endif
  printf("WS: %2d, ART: %s, D_cache: %s, I_cache: %s\n", FLASH->ACR & 0xF, (FLASH->ACR & 0x200)? "ON ":"OFF", ((SCB->CCR & 0x10000)? "ON ":"OFF"), ((SCB->CCR & 0x20000)? "ON ":"OFF"));	
  printf("System Clock: %9d, cycles: %d\n", HAL_RCC_GetSysClockFreq(),time_diff );
	printf("Running from address:   0x%08X \n", (uint32_t)main);
	printf("Data array_sdram at address: 0x%08X \n", (uint32_t)&array_sdram);
  printf("Data array_dtcm at address: 0x%08X \n", (uint32_t)&array_dtcm);
    
#ifdef __CC_ARM
	printf("Compiler KEIL, ver: %d ", __ARMCC_VERSION);
	#ifdef __TARGET_CPU_CORTEX_M7_FP_SP
	printf("and Core is M7 | \n\r");
	#endif
	#ifdef __TARGET_CPU_CORTEX_M4_FP
	printf("and Core is M4 | \n\r");
	#endif
#endif
#ifdef __ICCARM__
	printf("Compiler IAR, ver: %d \n\r", __VER__);
	#if __CORE__ == __ARM7M__
	printf("and Core is M7 | \n\r");
	#endif
	#if __CORE__ == __ARM4TM__
	printf("and Core is M4 | \n\r");
	#endif
#endif
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
static void Error_Handler(void)
{
  /* User may add here some code to deal with this error */
  while(1)
  {
  }
}


/**
  * @brief  CPU L1-Cache enable.
  * @param  None
  * @retval None
  */
static void CPU_CACHE_Enable(void)
{
  /* Enable I-Cache */
  SCB_EnableICache();

  /* Enable D-Cache */
  SCB_EnableDCache();
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
